%% scara_robot_starter.m
% 
% This Matlab script provides the starter code for the SCARA robot
% problem on Homework 2 in MEAM 520 at the University of Pennsylvania.
% The original was written by Professor Katherine J. Kuchenbecker in
% September of 2012. Students will modify this code to create their own
% script. Post questions on the class's Piazza forum.
% 
% Change the name of this file to replace "starter" with your PennKey.

%% SETUP

% Clear all variables from the workspace.
clear all

% Clear the console, so you can more easily find any errors that may occur.
clc

% Input your name as a string.
student_name = 'PUT YOUR NAME HERE';

% Define our time vector.
tStart = 0;   % The time at which the simulation starts, in seconds.
tStep = 0.04; % The simulation's time step, in seconds.
tEnd = 10;    % The time at which the simulation ends, in seconds.
t = (tStart:tStep:tEnd)';  % The time vector (a column vector).

% Set whether to animate the robot's movement and how much to slow it down.
pause on;  % Set this to off if you don't want to watch the animation.
GraphingTimeDelay = 0.001; % The length of time that Matlab should pause between positions when graphing, if at all, in seconds.


%% MOTION MODES

% 0 makes the robot sit still with all joint coordinates at zero.
% 1 makes only joint 1 move.
% 2 makes only joint 2 move.
% 3 makes only joint 3 move.
% 4 makes all of the joints move in an interesting pattern of your choosing.

% Set the mode of the robot motion here.
motion_mode = 1;

% Define the joint coordinates given the selected motion mode.  Here we are
% setting the values for theta1, theta2, and d3.  Each one must be a column
% vector of the same dimensions as t.
switch motion_mode
    case 0
        disp('The robot should be stationary.')
        theta1_history = zeros(length(t),1); % Set theta1 to zero for all time.
        theta2_history = 0*t; % Set theta2 to zero for all time (simpler method).
        d3_history = 0*t; % Set d3 to zero for all time.
    case 1
        disp('The robot should be moving only joint 1.')
        theta1_history = 1+.5*sin(1.3*t); % Set theta1 to an interesting function.
        theta2_history = 0*t; % Set theta2 to zero for all time.
        d3_history = 0*t; % Set d3 to zero for all time.
    case 2
        disp('The robot should be moving only joint 2.')
        % You should fill in what needs to be here.
    case 3
        disp('The robot should be moving only joint 3.')
        % You should fill in what needs to be here.
    case 4
        disp('The robot should be moving all of its joints.')
        % You should fill in what needs to be here.
    otherwise
        error('Error! Unknown motion_mode')
end

%% ROBOT PARAMETERS

% This problem is about the first three joints (RRP) of a SCARA
% manipulator.  This robot's forward kinematics are worked out on pages 91
% to 93 of the SHV textbook, though we are ignoring the fourth joint (the
% wrist).  You are welcome to use the book's solution to solve this part of
% the homework.

% Define robot link lengths.
a1 = 1.0; % Distance between joints 1 and 2, in meters.
a2 = 0.7; % Distance between joints 2 and 3, in meters.

%% SIMULATION

% Notify the user that we're starting the animation.
disp('Starting the animation.')

% Show a message to explain how to cancel the simulation and graphing.
disp('Click in this window and press control-c to stop the code.')

% Initialize a matrix to hold the position of the robot's tip over time.
% The first row is the x-coordinate, second is y, and third is z, all in
% the base frame.  You are welcome to make this 4 rows if you want to use
% the homogeneous representation for points.  It has the same number of
% columns as t has rows, one tip position for every time step in the
% simulation.  We keep track of this history so we can trace out the
% trajectory of where the robot's tip has been.
tip_history = zeros(3,length(t));

% Step through the time vector to animate the robot.
for i = 1:length(t)
    
    % Pull the current values of theta1, theta2, and d3 from their histories.
    theta1 = theta1_history(i);
    theta2 = theta2_history(i);
    d3 = d3_history(i);

    % Do your calculations. If you do things in a straightforward way, all
    % of your code should be between the two lines of stars.
    % *******************************************************************
    
    % Put your calculations here!
       
    % If your code runs but you can't see anything in your plot, comment
    % out or modify the axis([...]) command in the plot section below.
        
    % Put the points together.
    points_to_plot = [[0 0 -2]' [0 0 0]' [.5 0 1]' [2 0 1]' [theta1 theta2 d3]'];
    
    % All your calculations should be done by here.  Your list of robot
    % points to plot should be in points_to_plot, going from the base out
    % to the tip.  Each column represents the x, y, and z coordinates of
    % one point.  The last column of points_to_plot should be the position
    % of the tip of the robot.
    % *******************************************************************
    
    % Grab the final plotted point for the trajectory graph.
    tip_history(:,i) = points_to_plot(:,end);

    % Open figure 1.
    figure(1);

    % Check if this is the first time we are plotting.
    if (i == 1)
        % The first time, plot the robot points and keep a handle to the plot.
        % This is a 3D plot with dots at the points and lines connecting
        % neighboring points, made thicker, with big dots, in dark gray.
        hrobot = plot3(points_to_plot(1,:),points_to_plot(2,:),points_to_plot(3,:),'.-','linewidth',5,'markersize',20,'color',[.3 .3 .3]);
        
        % Also plot the tip position of the robot, using hold on and hold
        % off, also keeping a handle to the plot so we can update the data
        % points later.
        hold on;
        htip = plot3(tip_history(1,i),tip_history(2,i),tip_history(3,i),'r.');
        hold off;

        % Label the axes.
        xlabel('X (m)');
        ylabel('Y (m)');
        zlabel('Z (m)');

        % Turn on the grid and the box.
        grid on;
        box on;

        % Set the axis limits.
        axis([-2 2 -2 2 -2 2])

        % Set the axis properties for 3D visualization, which makes one
        % unit the same in every direction, and enables rotation.
        axis vis3d;

        % Put text on the plot to show how much time has elapsed.  The text
        % is centered.
        htime = text(1,1,1,sprintf('t = %.2f s',t(i)),'horizontalAlignment','center');

        % Add a title including the student's name.
        title(['SCARA Robot by ' student_name]);
    else
        % Once the animation has been set up, we don't need to reformat the
        % whole plot.  We just set the data to the correct new values for
        % the robot animation, the tip history, and the text showing the
        % elapsed time.
        set(hrobot,'xdata',points_to_plot(1,:),'ydata',points_to_plot(2,:),'zdata',points_to_plot(3,:))
        set(htip,'xdata',tip_history(1,1:i),'ydata',tip_history(2,1:i),'zdata',tip_history(3,1:i))
        set(htime,'string', (sprintf('t = %.2f s',t(i))));
    end
    
    % Pause for a short duration so that the viewer can watch animation evolve over time.
    pause(GraphingTimeDelay)
    
end

disp('Done with the animation.')
